@@ -1,6 +1,6 @@
 /* pkl-ast.h - Abstract Syntax Tree for Poke.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -744,11 +744,17 @@ pkl_ast_node pkl_ast_make_indexer (pkl_ast ast,
                                      pkl_ast_node index);
 
 /* PKL_AST_STRUCT_REF nodes represent references to a struct
-   element.  */
+   element.
+
+   If INDIRECTION_P is nonzero, then this node represents the indirection
+   of the struct element instead of a reference to the field itself.  In
+   these cases, the element must be of type referring offset.  */
 
 #define PKL_AST_STRUCT_REF_STRUCT(AST) ((AST)->sref.sct)
 #define PKL_AST_STRUCT_REF_IDENTIFIER(AST) ((AST)->sref.identifier)
 #define PKL_AST_STRUCT_REF_IS_PARENTHESIZED(AST) ((AST)->sref.is_parenthesized)
+#define PKL_AST_STRUCT_REF_INDIRECTION_P(AST) ((AST)->sref.indirection_p)
+#define PKL_AST_STRUCT_REF_ORIG_TYPE(AST) ((AST)->sref.orig_type)
 
 struct pkl_ast_struct_ref
 {
@@ -756,7 +762,9 @@ struct pkl_ast_struct_ref
 
   union pkl_ast_node *sct;
   union pkl_ast_node *identifier;
+  union pkl_ast_node *orig_type;
   int is_parenthesized;
+  int indirection_p;
 };
 
 pkl_ast_node pkl_ast_make_struct_ref (pkl_ast ast,
@@ -889,8 +897,11 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    is controlled in the pass manager.
 
    In integral types, SIGNED is 1 if the type denotes a signed numeric
-   type.  In non-integral types SIGNED is 0.  SIZE is the size in bits
-   of type.
+   type.  In non-integral types SIGNED_P is 0.  SIZE is the size in
+   bits of type, which shall be of the supported range (like [1,64]
+   for unsigned integers and [2,64] for signed integers.)  DYN_P is
+   not 0 if the integral type is dynamic, i.e. its size is not fixed
+   at compile-time like in int<*> and uint<*>.
 
    In array types, ETYPE is a PKL_AST_TYPE node reflecting the type of
    the elements stored in the array.  If the array type is bounded by
@@ -930,14 +941,21 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    When the size of a value of a given type can be determined at
    compile time, we say that such type is "complete".  Otherwise, we
    say that the type is "incomplete" and should be completed at
-   run-time.  */
+   run-time.
+
+   When a given type contains some sort of constraint that may fail at
+   either mapping or construction time, we say that such type is
+   "fallible".  Examples of such constraints are constraint
+   expressions in struct type fields or size bounders in arrays.  */
 
 #define PKL_AST_TYPE_CODE(AST) ((AST)->type.code)
 #define PKL_AST_TYPE_NAME(AST) ((AST)->type.name)
 #define PKL_AST_TYPE_COMPLETE(AST) ((AST)->type.complete)
+#define PKL_AST_TYPE_FALLIBLE(AST) ((AST)->type.fallible)
 #define PKL_AST_TYPE_COMPILED(AST) ((AST)->type.compiled)
 #define PKL_AST_TYPE_I_SIZE(AST) ((AST)->type.val.integral.size)
 #define PKL_AST_TYPE_I_SIGNED_P(AST) ((AST)->type.val.integral.signed_p)
+#define PKL_AST_TYPE_I_DYN_P(AST) ((AST)->type.val.integral.dyn_p)
 #define PKL_AST_TYPE_A_BOUND(AST) ((AST)->type.val.array.bound)
 #define PKL_AST_TYPE_A_ETYPE(AST) ((AST)->type.val.array.etype)
 #define PKL_AST_TYPE_A_CLOSURES(AST) ((AST)->type.val.array.closures)
@@ -964,6 +982,7 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
 #define PKL_AST_TYPE_S_ITYPE(AST) ((AST)->type.val.sct.itype)
 #define PKL_AST_TYPE_O_UNIT(AST) ((AST)->type.val.off.unit)
 #define PKL_AST_TYPE_O_BASE_TYPE(AST) ((AST)->type.val.off.base_type)
+#define PKL_AST_TYPE_O_REF_TYPE(AST) ((AST)->type.val.off.ref_type)
 #define PKL_AST_TYPE_F_RTYPE(AST) ((AST)->type.val.fun.rtype)
 #define PKL_AST_TYPE_F_NARG(AST) ((AST)->type.val.fun.narg)
 #define PKL_AST_TYPE_F_ARGS(AST) ((AST)->type.val.fun.args)
@@ -974,6 +993,10 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
 #define PKL_AST_TYPE_COMPLETE_YES 1
 #define PKL_AST_TYPE_COMPLETE_NO 2
 
+#define PKL_AST_TYPE_FALLIBLE_UNKNOWN 0
+#define PKL_AST_TYPE_FALLIBLE_YES 1
+#define PKL_AST_TYPE_FALLIBLE_NO 2
+
 struct pkl_ast_type
 {
   struct pkl_ast_common common;
@@ -982,6 +1005,7 @@ struct pkl_ast_type
   enum pkl_ast_type_code code;
   int complete;
   int compiled;
+  int fallible;
 
   union
   {
@@ -989,6 +1013,7 @@ struct pkl_ast_type
     {
       size_t size;
       int signed_p;
+      int dyn_p;
     } integral;
 
     struct
@@ -1020,6 +1045,7 @@ struct pkl_ast_type
     {
       union pkl_ast_node *unit;
       union pkl_ast_node *base_type;
+      union pkl_ast_node *ref_type;
     } off;
 
     struct
@@ -1051,7 +1077,7 @@ pkl_ast_node pkl_ast_make_struct_type (pkl_ast ast, size_t nelem, size_t nfield,
                                        int pinned_p, int union_p);
 
 pkl_ast_node pkl_ast_make_offset_type (pkl_ast ast, pkl_ast_node base_type,
-                                       pkl_ast_node unit);
+                                       pkl_ast_node unit, pkl_ast_node ref_type);
 
 pkl_ast_node pkl_ast_make_function_type (pkl_ast ast, pkl_ast_node rtype,
                                          size_t narg, pkl_ast_node args);
@@ -1076,6 +1102,7 @@ pkl_ast_node pkl_ast_sizeof_type (pkl_ast ast, pkl_ast_node type);
 size_t pkl_ast_sizeof_integral_type (pkl_ast_node type);
 
 int pkl_ast_type_is_complete (pkl_ast_node type);
+int pkl_ast_type_is_fallible (pkl_ast_node type);
 
 void pkl_print_type (FILE *out, pkl_ast_node type, int use_given_name);
 
@@ -1117,10 +1144,6 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
    NAME is PKL_AST_IDENTIFIER node containing the name in the
    association.
 
-   PREV_NAME is used in order to cache the name in the declaration
-   when the later is set to "" in re-definition of non-immutable
-   global objects.  See pkl-env.c to see how this field is used.
-
    INITIAL is the initial value of the entity.  The kind of node
    depends on what is being declared:
    - An expression node for a variable.
@@ -1128,6 +1151,13 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
    - A PKL_AST_FUNC for a function.
    - A constant expression node for an unit.
 
+   PREV_DECL is used to keep track of previous declaration with the
+   same name.  See pkl-env.c to see how this field is used.
+
+   REDECL_CHAIN is used to form a transient sibling relationships among
+   re-declared non-immutable global entities in a compilation session.
+   See pkl-env to see how this field is used.
+
    ORDER is the order of the declaration in its containing
    compile-time environment.  It is filled up when the declaration is
    registered in an environment.
@@ -1139,18 +1169,22 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
    STRUCT_FIELD_P indicates whether this declaration is for a variable
    corresponding to a struct field.
 
+   IN_STRUCT_P indicates whether this declaration is declared in the
+   body of a struct type.
+
    IMMUTABLE_P indicates whether this declaration can be redefined.
    Used when bootstrapping the compiler.  */
 
 #define PKL_AST_DECL_KIND(AST) ((AST)->decl.kind)
 #define PKL_AST_DECL_NAME(AST) ((AST)->decl.name)
-#define PKL_AST_DECL_PREV_NAME(AST) ((AST)->decl.prev_name)
 #define PKL_AST_DECL_INITIAL(AST) ((AST)->decl.initial)
+#define PKL_AST_DECL_PREV_DECL(AST) ((AST)->decl.prev_decl)
 #define PKL_AST_DECL_ORDER(AST) ((AST)->decl.order)
 #define PKL_AST_DECL_SOURCE(AST) ((AST)->decl.source)
 #define PKL_AST_DECL_STRUCT_FIELD_P(AST) ((AST)->decl.struct_field_p)
 #define PKL_AST_DECL_IN_STRUCT_P(AST) ((AST)->decl.in_struct_p)
 #define PKL_AST_DECL_IMMUTABLE_P(AST) ((AST)->decl.immutable_p)
+#define PKL_AST_DECL_REDECL_CHAIN(AST) ((AST)->decl.redecl_chain)
 
 #define PKL_AST_DECL_KIND_ANY 0
 #define PKL_AST_DECL_KIND_VAR 1
@@ -1167,9 +1201,10 @@ struct pkl_ast_decl
   int in_struct_p;
   int immutable_p;
   char *source;
-  char *prev_name;
   union pkl_ast_node *name;
   union pkl_ast_node *initial;
+  union pkl_ast_node *prev_decl;
+  union pkl_ast_node *redecl_chain;
   int order;
 };
 
@@ -1583,7 +1618,7 @@ pkl_ast_node pkl_ast_make_lambda (pkl_ast ast, pkl_ast_node function);
 
 /* PKL_AST_ASM nodes represent asm expressions.
 
-   TEMPLATE is a string node contaning assembler instructions.
+   TEMPLATE is a string node containing assembler instructions.
 
    TYPE is the type of the element that is left at the top of
    the stack once the program in TEMPLATE is executed.
@@ -1643,7 +1678,7 @@ pkl_ast_node pkl_ast_make_incrdecr (pkl_ast ast,
                                     pkl_ast_node exp, int order, int sign);
 
 
-/* PKL_AST_COMPOUND_STMT nodes represent compound statements in the
+/* PKL_AST_COMP_STMT nodes represent compound statements in the
    language.
 
    STMTS is a possibly empty chain of statements.
@@ -1661,10 +1696,6 @@ pkl_ast_node pkl_ast_make_incrdecr (pkl_ast ast,
 #define PKL_AST_COMP_STMT_FRAMELESS_P(AST) ((AST)->comp_stmt.frameless_p)
 
 #define PKL_AST_BUILTIN_NONE 0
-#define PKL_AST_BUILTIN_TERM_GET_COLOR 14
-#define PKL_AST_BUILTIN_TERM_SET_COLOR 15
-#define PKL_AST_BUILTIN_TERM_GET_BGCOLOR 16
-#define PKL_AST_BUILTIN_TERM_SET_BGCOLOR 17
 
 struct pkl_ast_comp_stmt
 {
@@ -2020,7 +2051,7 @@ pkl_ast_node pkl_ast_make_raise_stmt (pkl_ast ast, pkl_ast_node exp);
 /* PKL_AST_ASM nodes represent `asm' statements, which are used in
    order to inline PVM assembly in Poke programs.
 
-   TEMPLATE is a string node contaning assembler instructions.
+   TEMPLATE is a string node containing assembler instructions.
 
    INPUTS is a chain of expressions.  The result of these expressions
    will be pushed in the stack in the given order before executing
@@ -2162,8 +2193,15 @@ void pkl_ast_node_free_chain (pkl_ast_node ast);
 
 pkl_ast_node pkl_ast_reverse (pkl_ast_node ast);
 
+char *pkl_ast_format (pkl_ast_node ast);
 void pkl_ast_print (FILE *fp, pkl_ast_node ast);
 
-char *pkl_ast_format_loc (pkl_ast ast, pkl_ast_loc loc);
+/* Format location as a string.
+
+   FILENAME is the name of whatever source the Poke program is read from,
+   usually the name of a file.  If it is NULL, then the source part of
+   the location is formatted assuming <stdin>.  */
+
+char *pkl_ast_format_loc (const char *filename, pkl_ast_loc loc);
 
 #endif /* ! PKL_AST_H */
