@@ -1,6 +1,6 @@
 /* pkl-pass.c - Support for compiler passes.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -447,6 +447,8 @@ pkl_do_pass_1 (pkl_compiler compiler,
           case PKL_TYPE_OFFSET:
             PKL_PASS (PKL_AST_TYPE_O_BASE_TYPE (node));
             PKL_PASS (PKL_AST_TYPE_O_UNIT (node));
+            if (PKL_AST_TYPE_O_REF_TYPE (node))
+              PKL_PASS (PKL_AST_TYPE_O_REF_TYPE (node));
 
             break;
           case PKL_TYPE_INTEGRAL:
